//=============================================================================
// AVR231 App Note Bootloader
//=============================================================================

#include <avr/io.h>
#include <stdio.h>
#include <avr/interrupt.h>
#include "loader.h"
#include "bus.h"
#include "spm.h"
#include "crc.h"

unsigned char delay_cycles(unsigned long count){
	long i;
	for(i=0;i<count;i++){
		asm volatile("nop");
	}
}

//=============================================================================
// Starts here!

int main(void){
	cli();
	busInit();
	DDRC=0xff;
	PORTC=0x00;
	delay_cycles(30000);
	PORTC=0xff;
	delay_cycles(30000);
	// Loop forever (loops only if Application Section is damaged)
	for (;;){
		// Key pressed? Yes -> run the loader routine
		if (!(PINA & (1 << PA2))){
			// Typecast the function to a not-__C_TASK function (__C_TASK is
			// defined just to remove some unneeded register storing from the
			// loader() function)
			busSend('\r');
			busSend('L');
			((void (*)())loader)();	
		}else{
	
		#ifdef CRC_CHECK
		// Check that the Application Section contents is undamaged
		// by calculating the CRC of the whole memory.
		{
			unsigned int crc = 0;
			unsigned char *p = (unsigned char *)0x000000;
			unsigned char *n = (unsigned char *)MEM_SIZE;
	
			do
			{
				crc = CRC(crc, *p++);
			}
			while (--n);
	
			// Application Section damaged
			//   -> do not jump to Reset Vector of the Application Section
			if (crc) {
				DDRB = 0xff;
				do {
					PORTB ^= 0xff;
					delay_cycles( 1000000 );
				} while(1);
//				continue;
			}
		}
		#endif
		
		((void (*)())0x0000)();
		}
	}
}
