/*****************************************************************************
*
* Atmel Corporation
*
* File              : usart.c
* Compiler          : IAR EWAAVR 2.28a/3.10c
* Revision          : $Revision: 2.0 $
* Date              : $Date: Wednesday, February 15, 2006 09:52:02 UTC $
* Updated by        : $Author: raapeland $
*
* Support mail      : avr@atmel.com
*
* Supported devices : All devices with Bootloader Capabilities 
*                     , and at least 1-KB SRAM can be used.
*                     The example is written for  ATmega8
*                                                 ATmega16
*                                                 ATmega162
*                                                 ATmega169
*                                                 ATmega32
*                                                 ATmega64
*                                                 ATmega128
*
* AppNote           : AVR231 - AES Bootloader
*
* Description:	      USART interface.
*
******************************************************************************/


#include <avr/io.h>
#include <stdio.h>
#include "bus.h"

#ifndef F_CPU
#define F_CPU 14745600
#endif

#define UART_UBRR_CALC(BAUD_,FREQ_) ((FREQ_)/((BAUD_)*16L)-1)
#define UART_BAUD_RATE 9600


//=============================================================================
// Initialize bus interface

void busInit(void){
/*
	// 115200 baud @ 3.6864MHz
	UBRRL = 1;	
	// 9600 baud @ 3.6864MHz
//	UBRRL = 23;	
		
	// Enable Rx and Tx.
	UCSRB = (1 << RXEN) | (1 << TXEN);*/
	UCSR0B |= (1<<TXEN)|(1<<RXEN);			// UART TX/RX einschalten
	UCSR0C |= (0<<UMSEL0)|(3<<UCSZ00);	// Asynchron 8N1  <--- not needed by Bootloader?
  	UBRR0H = (uint8_t)( UART_UBRR_CALC( UART_BAUD_RATE, F_CPU ) >> 8 );
    UBRR0L = (uint8_t)UART_UBRR_CALC( UART_BAUD_RATE, F_CPU );


	UCSR1B |= (1<<TXEN)|(1<<RXEN);			// UART TX/RX einschalten
	UCSR1C |= (0<<UMSEL0)|(3<<UCSZ00);	// Asynchron 8N1  <--- not needed by Bootloader?
	UBRR1H = (uint8_t)( UART_UBRR_CALC( 115200, F_CPU ) >> 8 );
    UBRR1L = (uint8_t)UART_UBRR_CALC( 115200, F_CPU );

}


//=============================================================================
// Prepare bus (in this case, wait for the first byte to arrive).

void busPrepare(void){
	while (!(UCSR0A & (1 << RXC)));
}


//=============================================================================
// Reply byte

void busReplyByte(unsigned char data){
	while (!(UCSR0A & (1 << UDRE)));
	UDR0 = data;
}


//=============================================================================
// Receive byte

unsigned char busReceiveByte(void){
	busPrepare();
	return UDR0;
}


//=============================================================================
// Set bus to a busy state. Requires no action in USART.

void busBusy(void){
	return;
}

void busSend(unsigned char data){
    while (!(UCSR1A & (1<<UDRE))); /* warten bis Senden moeglich */
    UDR1 = data;                      /* sende Zeichen */
}

void busSendBoot(unsigned char data){
    while (!(UCSR0A & (1<<UDRE))); /* warten bis Senden moeglich */
    UDR0 = data;                      /* sende Zeichen */
}
